//written by: Joshua Devasagayaraj
//tested by: Joshua Devasagayaraj
//debugged by: Joshua Devasagayaraj


import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.io.InputStream;
import java.util.ArrayList;
import java.util.Iterator;

//Imports for Excel file reading. Requires the Apache POI library
import org.apache.poi.hssf.usermodel.HSSFRow;
import org.apache.poi.hssf.usermodel.HSSFSheet;
import org.apache.poi.hssf.usermodel.HSSFWorkbook;
import org.apache.poi.ss.usermodel.Row;

//SWT Imports
import org.eclipse.swt.SWT;
import org.eclipse.swt.custom.ScrolledComposite;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Display;
import org.eclipse.swt.widgets.FileDialog;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.List;
import org.eclipse.swt.widgets.ProgressBar;
import org.eclipse.swt.widgets.Shell;

/**
 * The Class WaiterGUI is the main class and contains the code for generating
 * the GUI as well as all the logic in handling table orders and menu items.
 */
public class WaiterGUI {

	/**
	 * The subclass entry is just a temporary class to organize visual entities
	 * in the GUI. The four buttons that correspond to the next four items in
	 * the queue all map to entries.
	 */
	class entry {

		/** The table number that this entry belongs to */
		public int tableID;

		/** The menu item list of the entry */
		public ArrayList<String> menuItemList;

		/** The table order number that the entry belongs to */
		public int tableOrder;

		/**
		 * Instantiates a new entry.
		 *
		 * @param tableID
		 *            the table id
		 * @param tableOrder
		 *            the table order
		 */
		public entry(final int tableID, final int tableOrder) {
			this.tableID = tableID;
			menuItemList = new ArrayList<String>();
			this.tableOrder = tableOrder;
		}

		/**
		 * Instantiates a new entry from another entry.
		 *
		 * @param ent
		 *            the ent
		 */
		public entry(final entry ent) {
			this.tableID = ent.tableID;
			this.menuItemList = ent.menuItemList;
			this.tableOrder = ent.tableOrder;
		}

	}

	/** The shell for the GUI */
	protected Shell shell;

	/** The menu item list containing the list of menu items ready to be queued. */
	ArrayList<MenuItem> menuItemList;

	/** The table order list containing the list of table orders in the system. */
	ArrayList<TableOrder> tableOrderList;

	/** The entries containing the current items displayed in the GUI */
	ArrayList<entry> entries;

	/** The temp entries of menu orders being built but not yet displayed */
	ArrayList<entry> tempEntries;

	/** The table button1. */
	private Button tableButton1;

	/** The table button2. */
	private Button tableButton2;

	/** The table button3. */
	private Button tableButton3;

	/** The table description1. */
	private Label tableDescription1;

	/** The table description2. */
	private Label tableDescription2;

	/** The table description3. */
	private Label tableDescription3;

	/** The table description4. */
	private Label tableDescription4;

	/** The table button4. */
	private Button tableButton4;

	/** The table list1. */
	private List tableList1;

	/** The table list2. */
	private List tableList2;

	/** The table list3. */
	private List tableList3;

	/** The table list4. */
	private List tableList4;

	/** The progress bar for the backlog. */
	private ProgressBar progressBar;

	/** The menu item list display for demo purposes. */
	private List menuItemListDisplay;

	/** The table order list display for demo purposes. */
	private List tableOrderListDisplay;

	/**
	 * The main method to launch the application.
	 *
	 * @param args
	 *            the arguments
	 */
	public static void main(final String[] args) {
		try {
			WaiterGUI window = new WaiterGUI();
			window.open();
		} catch (Exception e) {
			e.printStackTrace();
		}
	}

	/**
	 * Method to open the window.
	 */
	public final void open() {
		Display display = Display.getDefault();
		createContents();
		shell.open();
		shell.layout();
		while (!shell.isDisposed()) {
			if (!display.readAndDispatch()) {
				display.sleep();
			}
		}
	}

	/**
	 * This method will update the waiter's GUI by updating the buttons,
	 * descriptions and labels to reflect what is contained in the entries list.
	 * This means that it will update the buttons and information to what is
	 * currently ready to be queued.
	 */
	public final void updateGUI() {
		updateTableOrders();
		updateMenuItems();

		entry e;
		boolean show = false;
		if (entries.size() >= 1) {
			e = entries.get(0);
			show = true;
			tableButton1.setVisible(true);
			tableDescription1.setVisible(true);
			tableList1.setVisible(true);
			if (e.tableOrder != 0) {
				tableButton1.setText("Table " + e.tableID);
				tableDescription1.setText("Table Order: " + e.tableOrder);
				String[] arr = new String[e.menuItemList.size()];
				for (int i = 0; i < arr.length; i++) {
					arr[i] = e.menuItemList.get(i);
				}
				tableList1.setItems(arr);
			} else {
				tableButton1.setText("Table " + e.tableID);
				tableDescription1.setText("Individual Order");
				String[] arr = new String[1];
				arr[0] = e.menuItemList.get(0);
				tableList1.setItems(arr);
			}
		} else {
			tableButton1.setVisible(false);
			tableDescription1.setVisible(false);
			tableList1.setVisible(false);
		}

		if (entries.size() >= 2) {
			e = entries.get(1);
			tableButton2.setVisible(true);
			tableDescription2.setVisible(true);
			tableList2.setVisible(true);
			if (e.tableOrder != 0) {
				tableButton2.setText("Table " + e.tableID);
				tableDescription2.setText("Table Order: " + e.tableOrder);
				String[] arr = new String[e.menuItemList.size()];
				for (int i = 0; i < arr.length; i++) {
					arr[i] = e.menuItemList.get(i);
				}
				tableList2.setItems(arr);
			} else {
				tableButton2.setText("Table " + e.tableID);
				tableDescription2.setText("Individual Order");
				String[] arr = new String[1];
				arr[0] = e.menuItemList.get(0);
				tableList2.setItems(arr);
			}
		} else {
			tableButton2.setVisible(false);
			tableDescription2.setVisible(false);
			tableList2.setVisible(false);
		}

		if (entries.size() >= 3) {
			e = entries.get(2);
			tableButton3.setVisible(true);
			tableDescription3.setVisible(true);
			tableList3.setVisible(true);
			if (e.tableOrder != 0) {
				tableButton3.setText("Table " + e.tableID);
				tableDescription3.setText("Table Order: " + e.tableOrder);
				String[] arr = new String[e.menuItemList.size()];
				for (int i = 0; i < arr.length; i++) {
					arr[i] = e.menuItemList.get(i);
				}
				tableList3.setItems(arr);
			} else {
				tableButton3.setText("Table " + e.tableID);
				tableDescription3.setText("Individual Order");
				String[] arr = new String[1];
				arr[0] = e.menuItemList.get(0);
				tableList3.setItems(arr);
			}
		} else {
			tableButton3.setVisible(false);
			tableDescription3.setVisible(false);
			tableList3.setVisible(false);
		}

		if (entries.size() >= 4) {
			e = entries.get(3);
			tableButton4.setVisible(true);
			tableDescription4.setVisible(true);
			tableList4.setVisible(true);
			if (e.tableOrder != 0) {
				tableButton4.setText("Table " + e.tableID);
				tableDescription4.setText("Table Order: " + e.tableOrder);
				String[] arr = new String[e.menuItemList.size()];
				for (int i = 0; i < arr.length; i++) {
					arr[i] = e.menuItemList.get(i);
				}
				tableList4.setItems(arr);
			} else {
				tableButton4.setText("Table " + e.tableID);
				tableDescription4.setText("Individual Order");
				String[] arr = new String[1];
				arr[0] = e.menuItemList.get(0);
				tableList4.setItems(arr);
			}
		} else {
			tableButton4.setVisible(false);
			tableDescription4.setVisible(false);
			tableList4.setVisible(false);
		}

		if (show && entries.size() > 4) {
			progressBar.setVisible(true);
			progressBar.setSelection(100 * (entries.size() - 4) / 10);
		} else if (entries.size() <= 4) {
			progressBar.setSelection(0);
		}

		updateTableOrders();
		updateMenuItems();

	}

	/**
	 * Updates the list of table orders that are currently in the GUI.
	 */
	final void updateTableOrders() {

		String[] arr = new String[tableOrderList.size()];
		for (int i = 0; i < tableOrderList.size(); i++) {
			arr[i] = tableOrderList.get(i).toString();
		}

		tableOrderListDisplay.setItems(arr);
		tableOrderListDisplay.redraw();

	}

	/**
	 * Updates the list of menu items that are currently in the GUI.
	 */
	final void updateMenuItems() {
		String[] arr = new String[menuItemList.size()];
		for (int i = 0; i < menuItemList.size(); i++) {
			arr[i] = menuItemList.get(i).toString();
		}

		menuItemListDisplay.setItems(arr);
		tableOrderListDisplay.redraw();
	}

	/**
	 * Creates the contents of the GUI window. Initializes the buttons, labels,
	 * lists, etc. Implementations for the buttons are also contained here.
	 * Comments can be found above each implementation.
	 */
	protected final void createContents() {

		tableOrderList = new ArrayList<TableOrder>();
		menuItemList = new ArrayList<MenuItem>();
		entries = new ArrayList<entry>();
		tempEntries = new ArrayList<entry>();

		shell = new Shell();
		shell.setSize(1314, 712);
		shell.setText("SWT Application");

		Label label = new Label(shell, SWT.SEPARATOR | SWT.VERTICAL);
		label.setBounds(707, 48, 2, 590);

		tableButton1 = new Button(shell, SWT.FLAT);
		tableButton1.addSelectionListener(new SelectionAdapter() {
			@Override
			public void widgetSelected(final SelectionEvent e) {
				System.out.println("ENtries Size: " + entries.size());
				if (entries.size() >= 1) {
					entries.remove(0);
					updateGUI();
				}
			}
		});
		tableButton1.setText("Table: #");
		tableButton1.setBounds(21, 95, 145, 57);
		tableButton1.setVisible(false);

		tableButton2 = new Button(shell, SWT.FLAT);
		tableButton2.addSelectionListener(new SelectionAdapter() {
			@Override
			public void widgetSelected(final SelectionEvent e) {
				if (entries.size() >= 2) {
					entries.remove(1);
					updateGUI();
				}
			}
		});
		tableButton2.setText("Table: #");
		tableButton2.setBounds(184, 95, 145, 57);
		tableButton2.setVisible(false);

		tableButton3 = new Button(shell, SWT.FLAT);
		tableButton3.addSelectionListener(new SelectionAdapter() {
			@Override
			public void widgetSelected(final SelectionEvent e) {
				if (entries.size() >= 3) {
					entries.remove(2);
					updateGUI();
				}
			}
		});
		tableButton3.setText("Table: #");
		tableButton3.setBounds(347, 95, 145, 57);
		tableButton3.setVisible(false);

		/*
		 *This is the implementation for the queue button. It will take a menu item
		 *from the menu item list as this is is the list of items that are ready
		 *to be queued. It will then apply the queuing algorithm discussed in the report
		 *and either accumulate the table order associated with it or send it to the ready
		 *queue as an individual order. If when adding to the table order, the table orders'
		 *items in its entirety becomes ready, the order will then be queued.
		 */
		Button queueButton = new Button(shell, SWT.NONE);
		queueButton.addSelectionListener(new SelectionAdapter() {
			@Override
			public void widgetSelected(final SelectionEvent e) {
				if (menuItemList.isEmpty()) {
					return;
				}
				MenuItem i = menuItemList.remove(0);
				boolean found = false;
				Iterator<TableOrder> iter = tableOrderList.iterator();
				while (iter.hasNext()) {
					TableOrder t = iter.next();
					for (String item : t.menuItemIDs.split(",")) {
						if (i.id == Integer.parseInt(item)) {
							found = true;

							t.currentNumMenuItems++;
							if (t.currentNumMenuItems == t.totalNumMenuItems) {

								for (entry ent : tempEntries) {
									if (ent.tableID == i.tableNum) {
										ent.menuItemList.add(i.name);
										entries.add(new entry(ent));
									}
								}
								iter.remove();
							}

							else {
								boolean foundTempEntry = false;
								for (entry ent : tempEntries) {
									if (ent.tableID == i.tableNum) {
										foundTempEntry = true;
										ent.menuItemList.add(i.name);
									}

								}
								if (!foundTempEntry) {
									entry toBeAdded = new entry(t.tableNum,
											t.id);
									toBeAdded.menuItemList.add(i.name);
									tempEntries.add(toBeAdded);

								}
							}
						}

					}
				}

				if (!found) {
					entry toBeAdded = new entry(i.tableNum, 0);
					toBeAdded.menuItemList.add(i.name);
					entries.add(toBeAdded);
					System.out.println("added new entry");
				}

				updateGUI();
			}
		});

		queueButton.setBounds(1165, 600, 111, 47);
		queueButton.setText("Queue");

		final Label menuItemQueueLabel = new Label(shell, SWT.NONE);
		menuItemQueueLabel.setBounds(1032, 581, 55, 15);
		menuItemQueueLabel.setText("File:");

		ScrolledComposite scrolledComposite = new ScrolledComposite(shell,
				SWT.BORDER | SWT.H_SCROLL | SWT.V_SCROLL);
		scrolledComposite.setBounds(733, 95, 228, 478);
		scrolledComposite.setExpandHorizontal(true);
		scrolledComposite.setExpandVertical(true);

		tableOrderListDisplay = new List(scrolledComposite, SWT.BORDER);
		tableOrderListDisplay.setItems(new String[] { "" });
		scrolledComposite.setContent(tableOrderListDisplay);
		scrolledComposite.setMinSize(tableOrderListDisplay.computeSize(
				SWT.DEFAULT, SWT.DEFAULT));

		ScrolledComposite scrolledComposite_1 = new ScrolledComposite(shell,
				SWT.BORDER | SWT.H_SCROLL | SWT.V_SCROLL);
		scrolledComposite_1.setBounds(1032, 95, 244, 478);
		scrolledComposite_1.setExpandHorizontal(true);
		scrolledComposite_1.setExpandVertical(true);

		menuItemListDisplay = new List(scrolledComposite_1, SWT.BORDER);
		menuItemListDisplay.setItems(new String[] { "" });
		scrolledComposite_1.setContent(menuItemListDisplay);
		scrolledComposite_1.setMinSize(menuItemListDisplay.computeSize(
				SWT.DEFAULT, SWT.DEFAULT));

		/*
		 * The following is the implementation for the read queue buttons which
		 * will read in a properly formated excel file containing menu itemsand
		 * creates menu item objects accordingly. It will then update the listof
		 * menu items to reflect this.
		 */
		Button readQueueButton = new Button(shell, SWT.NONE);
		readQueueButton.addSelectionListener(new SelectionAdapter() {
			@Override
			public void widgetSelected(final SelectionEvent e) {

				FileDialog fd = new FileDialog(shell, SWT.OPEN);
				fd.setText("Open");
				fd.setFilterPath("C:/");
				String[] filterExt = { "*.xls" };
				fd.setFilterExtensions(filterExt);
				String selected = fd.open();
				System.out.println(selected);

				if (selected == null) {
					menuItemQueueLabel.setText("No File Selected!");
				}

				else {
					InputStream input;
					try {
						input = new FileInputStream(selected);
						HSSFWorkbook wb = new HSSFWorkbook(input);
						HSSFSheet ws = wb.getSheetAt(0);
						Iterator<Row> iter = ws.rowIterator();
						iter.next();
						while (iter.hasNext()) {
							HSSFRow row = (HSSFRow) iter.next();
							menuItemList.add(new MenuItem((int) row.getCell(0)
									.getNumericCellValue(), row.getCell(1)
									.getStringCellValue(), (int) row.getCell(2)
									.getNumericCellValue()));
						}

						updateMenuItems();

					} catch (FileNotFoundException e1) {
						menuItemQueueLabel.setText("Could not find file");
						e1.printStackTrace();
					} catch (IOException e1) {
						menuItemQueueLabel.setText("I/O exception occured.");
						e1.printStackTrace();
					}
				}
			}
		});
		readQueueButton.setBounds(1032, 600, 100, 47);
		readQueueButton.setText("Read Queue");

		Label lblWaiterGui = new Label(shell, SWT.NONE);
		lblWaiterGui.setBounds(253, 40, 109, 30);
		lblWaiterGui.setText("Waiter GUI");

		tableButton4 = new Button(shell, SWT.FLAT);
		tableButton4.addSelectionListener(new SelectionAdapter() {
			@Override
			public void widgetSelected(final SelectionEvent e) {
				if (entries.size() >= 4) {
					entries.remove(3);
					updateGUI();
				}
			}
		});
		tableButton4.setText("Table: #");
		tableButton4.setBounds(509, 95, 145, 57);
		tableButton4.setVisible(false);

		Label lblReadyToBe = new Label(shell, SWT.NONE);
		lblReadyToBe.setText("Ready To be Queued");
		lblReadyToBe.setBounds(1064, 48, 212, 30);

		tableList1 = new List(shell, SWT.BORDER);
		tableList1.setBounds(27, 225, 139, 186);
		tableList1.setVisible(false);

		tableList2 = new List(shell, SWT.BORDER);
		tableList2.setBounds(184, 225, 139, 186);
		tableList2.setVisible(false);

		tableList3 = new List(shell, SWT.BORDER);
		tableList3.setBounds(347, 225, 139, 186);
		tableList3.setVisible(false);

		tableList4 = new List(shell, SWT.BORDER);
		tableList4.setBounds(509, 225, 139, 186);
		tableList4.setVisible(false);

		tableDescription1 = new Label(shell, SWT.NONE);
		tableDescription1.setBounds(27, 186, 139, 22);
		tableDescription1.setText("Table Order #");
		tableDescription1.setVisible(false);

		tableDescription2 = new Label(shell, SWT.NONE);
		tableDescription2.setText("Table Order #");
		tableDescription2.setBounds(184, 186, 139, 22);
		tableDescription2.setVisible(false);

		tableDescription3 = new Label(shell, SWT.NONE);
		tableDescription3.setText("Table Order #");
		tableDescription3.setBounds(347, 186, 139, 22);
		tableDescription3.setVisible(false);

		tableDescription4 = new Label(shell, SWT.NONE);
		tableDescription4.setText("Table Order #");
		tableDescription4.setBounds(509, 186, 139, 22);
		tableDescription4.setVisible(false);

		Label label_1 = new Label(shell, SWT.SEPARATOR);
		label_1.setBounds(999, 48, 2, 590);

		Label label_2 = new Label(shell, SWT.NONE);
		label_2.setText("For Demo Purposes:");
		label_2.setBounds(891, 10, 212, 30);

		Label lblCurrentTableOrders = new Label(shell, SWT.NONE);
		lblCurrentTableOrders.setText("Table Order Wait Queue");
		lblCurrentTableOrders.setBounds(747, 48, 212, 30);

		final Label tableOrdersFileLabel = new Label(shell, SWT.NONE);
		tableOrdersFileLabel.setBounds(733, 579, 55, 15);
		tableOrdersFileLabel.setText("File:");

		/*
		 * The following is the implementation for the read table orders button
		 * which will read in a properly formatted excel file containing a list
		 * of table orders and creates a list of table order objects
		 * accordingly. It will then update the list of table orders in the GUI.
		 */
		Button readTableOrders = new Button(shell, SWT.NONE);
		readTableOrders.addSelectionListener(new SelectionAdapter() {
			@Override
			public void widgetSelected(final SelectionEvent e) {
				FileDialog fd = new FileDialog(shell, SWT.OPEN);
				fd.setText("Open");
				fd.setFilterPath("C:/");
				String[] filterExt = { "*.xls" };
				fd.setFilterExtensions(filterExt);
				String selected = fd.open();
				System.out.println(selected);

				if (selected == null) {
					tableOrdersFileLabel.setText("No File Selected!");
				}

				else {
					InputStream input;
					try {
						input = new FileInputStream(selected);
						HSSFWorkbook wb = new HSSFWorkbook(input);
						HSSFSheet ws = wb.getSheetAt(0);
						Iterator<Row> iter = ws.rowIterator();
						iter.next();
						while (iter.hasNext()) {
							HSSFRow row = (HSSFRow) iter.next();
							tableOrderList.add(new TableOrder((int) row
									.getCell(0).getNumericCellValue(),
									(int) row.getCell(1).getNumericCellValue(),
									(int) row.getCell(2).getNumericCellValue(),
									row.getCell(3).getStringCellValue()));
						}

						updateTableOrders();

					} catch (FileNotFoundException e1) {
						tableOrdersFileLabel.setText("Could not find file");
						e1.printStackTrace();
					} catch (IOException e1) {
						tableOrdersFileLabel.setText("I/O exception occured.");
						e1.printStackTrace();
					}
				}

			}
		});
		readTableOrders.setText("Read Table Orders");
		readTableOrders.setBounds(733, 600, 118, 47);

		Label lblAssistanceRequested = new Label(shell, SWT.NONE);
		lblAssistanceRequested.setEnabled(false);
		lblAssistanceRequested.setText("Assistance Requested");
		lblAssistanceRequested.setBounds(21, 452, 212, 30);

		Button btnNewButton_3 = new Button(shell, SWT.NONE);
		btnNewButton_3.setEnabled(false);
		btnNewButton_3.setBounds(21, 505, 125, 57);

		Button button_1 = new Button(shell, SWT.NONE);
		button_1.setEnabled(false);
		button_1.setBounds(152, 505, 125, 57);

		Button button_2 = new Button(shell, SWT.NONE);
		button_2.setEnabled(false);
		button_2.setBounds(283, 505, 125, 57);

		Button button_3 = new Button(shell, SWT.NONE);
		button_3.setEnabled(false);
		button_3.setBounds(414, 505, 125, 57);

		Button button_4 = new Button(shell, SWT.NONE);
		button_4.setEnabled(false);
		button_4.setBounds(545, 505, 125, 57);

		Button button_5 = new Button(shell, SWT.NONE);
		button_5.setEnabled(false);
		button_5.setBounds(21, 581, 125, 57);

		Button button_6 = new Button(shell, SWT.NONE);
		button_6.setEnabled(false);
		button_6.setBounds(152, 581, 125, 57);

		Button button_7 = new Button(shell, SWT.NONE);
		button_7.setEnabled(false);
		button_7.setBounds(283, 581, 125, 57);

		Button button_8 = new Button(shell, SWT.NONE);
		button_8.setEnabled(false);
		button_8.setBounds(414, 581, 125, 57);

		Button button_9 = new Button(shell, SWT.NONE);
		button_9.setEnabled(false);
		button_9.setBounds(545, 581, 125, 57);

		progressBar = new ProgressBar(shell, SWT.NONE);
		progressBar.setBounds(82, 429, 381, 17);
		progressBar.setVisible(false);

		Label lblBacklog = new Label(shell, SWT.NONE);
		lblBacklog.setBounds(21, 431, 55, 15);
		lblBacklog.setText("Backlog:");

	}
}
