// written by: Joshua Devasagayaraj
// tested by: Joshua Devasagayaraj
// debugged by:  Joshua Devasagayaraj

import java.io.BufferedInputStream;
import java.io.BufferedOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.ObjectInputStream;
import java.io.ObjectOutputStream;
import java.io.OutputStream;
import java.net.ServerSocket;
import java.net.Socket;
import java.util.ArrayList;

// TODO: Auto-generated Javadoc
/**
 * The Class Controller is the main class for controlling where messages go, logging interactions between classes
 * and handling inventory changes.
 */
public class Controller implements Runnable {

	/** The Constant chefIP. */
	public static final String chefIP = "localhost";

	/** The Constant chefPort. */
	public static final int chefPort = 15000;

	/** The Constant waiterIP. */
	public static final String waiterIP = "localhost";

	/** The Constant waiterPort. */
	public static final int waiterPort = 16000;

	/** The client socket. */
	Socket clientSocket;

	/** The socket input stream. */
	InputStream socketInputStream;

	/** The socket output stream. */
	OutputStream socketOutputStream;

	/** The object output stream. */
	ObjectOutputStream objectOutputStream;

	/** The object input stream. */
	ObjectInputStream objectInputStream;

	/** The menu. */
	Message.DATA_Menu menu;

	/** The inventory. */
	Message.DATA_Inventory inventory;

	/**
	 * Instantiates a new controller.
	 *
	 * @param clientSocket the client socket
	 * @param menu the menu
	 * @param inventory the inventory
	 */
	public Controller(Socket clientSocket, Message.DATA_Menu menu,
			Message.DATA_Inventory inventory) {
		this.clientSocket = clientSocket;
		this.menu = menu;
		this.inventory = inventory;

	}

	/**
	 * Adds the menu.
	 *
	 * @param item the item
	 */
	public void addMenu(Message.DATA_MenuItem item) {
		synchronized (item) {
			menu.menuItems.add(item);
		}
	}

	/**
	 * Check inventory to see if an order can be placed, if it can, then subtract the amounts from the inventory.
	 *
	 * @param order the order
	 * @return true, if successful
	 */
	public boolean checkInventory(Message.DATA_TableOrder order) {

		ArrayList<Message.DATA_Ingredient> temp = new ArrayList<Message.DATA_Ingredient>();
		int found = 0;
		for (Message.DATA_MenuItem item : order.menuItemList) {

			for (Message.DATA_Ingredient i : item.menuIngredientList) {
				for (Message.DATA_Ingredient tempi : temp) {
					if (tempi.name.compareTo(i.name) == 0) {
						found = 1;
					}
				}
				if (found == 0) {
					temp.add(new Message.DATA_Ingredient(i.name, i.amount));
				}
				found = 0;
			}
		}
		System.out.println(temp.size());
		System.out.flush();
		boolean ret = true;

		for (Message.DATA_Ingredient i : temp) {

			for (Message.DATA_Ingredient i2 : inventory.inventoryIngredientList) {
				if (i.name.compareTo(i2.name) == 0) {
					if (i.amount > i2.amount) {
						ret = false;
					} else {
						i2.amount -= i.amount;
					}
				}
			}
		}

		return ret;
	}

	/**
	 * In, the run method, it will take the message that it received and determine what to do with it depending
	 * on the type of message.
	 */
	public void run() {

		try {
			socketInputStream = clientSocket.getInputStream();
			socketOutputStream = clientSocket.getOutputStream();
			objectOutputStream = new ObjectOutputStream(
					new BufferedOutputStream(socketOutputStream));
			objectOutputStream.flush();
			this.objectInputStream = new ObjectInputStream(
					new BufferedInputStream(socketInputStream));

			Object o = objectInputStream.readObject();

			if (o instanceof Message.DATA_TableOrder) {

				Message.DATA_TableOrder order = (Message.DATA_TableOrder) o;

				// If inventory test passes, send confirmation and pass message
				// to chef.
				if (checkInventory(order)) {
					Communicator.send(o, chefIP, chefPort);
					Communicator.send(o, waiterIP, waiterPort);
					objectOutputStream
							.writeObject(new Message.CONFIRM_PLACE_ORDER());
					objectOutputStream.flush();
					objectOutputStream.close();
					objectInputStream.close();

					JLogger.log("Customer at table " + order.tableID
							+ " Sucessfully ordered, adjusted inventory. ");
				}
				// else send an error message back to customer.
				else {
					objectOutputStream
							.writeObject(new Message.ERROR_CANNOT_COOK_ORDER());
					objectOutputStream.flush();
					objectOutputStream.close();
					JLogger.log("Customer cannot order because inventory is low");
				}
			} else if (o instanceof Message.DATA_MenuItem) {

				// adjust inventory and send to waiter.
				Message.DATA_MenuItem item = (Message.DATA_MenuItem) o;
				Communicator.send(item, waiterIP, waiterPort);
				JLogger.log("Chef finished cooking " + item.name
						+ ". Sending to waiter.");

			} else if (o instanceof Message.Request_Inventory) {
				synchronized (inventory) {
					objectOutputStream.writeObject(inventory);
					objectOutputStream.flush();
				}
				// TODO: send inventory back.
			} else if (o instanceof Message.Request_Menu) {
				synchronized (menu) {
					objectOutputStream.writeObject(menu);
					objectOutputStream.flush();
					objectOutputStream.close();
				}
				// TODO: send menu back.
			} else if (o instanceof Message.Request_Waiter) {
				Communicator.send(o, waiterIP, waiterPort);
				objectOutputStream.flush();
				objectOutputStream.close();
				// TODO: send to waiter.
			} else if (o instanceof Message.Request_AddIngredient) {

				synchronized (inventory) {
					inventory.inventoryIngredientList
							.add(((Message.Request_AddIngredient) o).ingredient);
					objectOutputStream.writeObject(inventory);
					objectOutputStream.flush();
				}

				JLogger.log("Manager added ingredient: "
						+ ((Message.Request_AddIngredient) o).ingredient.name);

			} else if (o instanceof Message.Request_AddMenuItem) {

				synchronized (menu) {
					menu.menuItems.add(((Message.Request_AddMenuItem) o).item);
					objectOutputStream.writeObject(menu);
					objectOutputStream.flush();
					objectOutputStream.close();
				}

				JLogger.log("Manager added Menu Item: "
						+ ((Message.Request_AddMenuItem) o).item.name);

			} else {
				System.out.println("Recieved Unknown message!");
			}

		} catch (IOException e) {
			e.printStackTrace();
		} catch (ClassNotFoundException e) {
			e.printStackTrace();
		}

	}

	/**
	 * The main method will create the default menu and constantly listen for connections and messages
	 *
	 * @param args the arguments
	 */
	public static void main(String[] args) {

		Message.DATA_Menu menu = new Message.DATA_Menu();

		Message.DATA_Inventory inventory = new Message.DATA_Inventory();

		Message.DATA_Ingredient ingredient1 = new Message.DATA_Ingredient(
				"Bun", 500);
		Message.DATA_Ingredient ingredient2 = new Message.DATA_Ingredient(
				"CheeseSlice", 500);
		Message.DATA_Ingredient ingredient3 = new Message.DATA_Ingredient(
				"Patty", 500);
		Message.DATA_Ingredient ingredient4 = new Message.DATA_Ingredient(
				"Lettuce", 500);
		Message.DATA_Ingredient ingredient5 = new Message.DATA_Ingredient(
				"Turkey", 500);
		Message.DATA_Ingredient ingredient6 = new Message.DATA_Ingredient(
				"Beef", 500);
		Message.DATA_Ingredient ingredient7 = new Message.DATA_Ingredient(
				"Bacon", 500);
		Message.DATA_Ingredient ingredient8 = new Message.DATA_Ingredient(
				"Tuna", 500);
		Message.DATA_Ingredient ingredient9 = new Message.DATA_Ingredient(
				"Tomatoes", 500);
		Message.DATA_Ingredient ingredient10 = new Message.DATA_Ingredient(
				"WheatBowTiePasta", 500);
		Message.DATA_Ingredient ingredient11 = new Message.DATA_Ingredient(
				"Spice", 500);
		Message.DATA_Ingredient ingredient12 = new Message.DATA_Ingredient(
				"ChocolateSprinkles", 500);
		Message.DATA_Ingredient ingredient13 = new Message.DATA_Ingredient(
				"Oreos", 500);
		Message.DATA_Ingredient ingredient14 = new Message.DATA_Ingredient(
				"WhipCream", 500);
		Message.DATA_Ingredient ingredient15 = new Message.DATA_Ingredient(
				"BeefStock", 500);
		Message.DATA_Ingredient ingredient16 = new Message.DATA_Ingredient(
				"GarlicPieces", 500);
		Message.DATA_Ingredient ingredient17 = new Message.DATA_Ingredient(
				"Salt/Pepper", 500);

		ArrayList<Message.DATA_Ingredient> inventoryIngredientList = new ArrayList<Message.DATA_Ingredient>();

		inventoryIngredientList.add(ingredient1);
		inventoryIngredientList.add(ingredient2);
		inventoryIngredientList.add(ingredient3);
		inventoryIngredientList.add(ingredient4);
		inventoryIngredientList.add(ingredient5);
		inventoryIngredientList.add(ingredient6);
		inventoryIngredientList.add(ingredient7);
		inventoryIngredientList.add(ingredient8);
		inventoryIngredientList.add(ingredient9);
		inventoryIngredientList.add(ingredient10);
		inventoryIngredientList.add(ingredient11);
		inventoryIngredientList.add(ingredient12);
		inventoryIngredientList.add(ingredient13);
		inventoryIngredientList.add(ingredient14);

		inventory.inventoryIngredientList = inventoryIngredientList;

		// ///////////////////////////////////////////////////////////////////////////////////////
		Message.DATA_MenuItem burger = new Message.DATA_MenuItem("Burger",
				11.00, 13, 0, 0);

		Message.DATA_Ingredient burger1 = new Message.DATA_Ingredient(
				"CheeseSlice", 3);
		Message.DATA_Ingredient burger2 = new Message.DATA_Ingredient("Bun", 1);
		Message.DATA_Ingredient burger3 = new Message.DATA_Ingredient("Patty",
				1);
		Message.DATA_Ingredient burger4 = new Message.DATA_Ingredient(
				"Lettuce", 1);

		ArrayList<Message.DATA_Ingredient> Cheeseburger = new ArrayList<Message.DATA_Ingredient>();
		Cheeseburger.add(burger1);
		Cheeseburger.add(burger2);
		Cheeseburger.add(burger3);
		Cheeseburger.add(burger4);

		burger.menuIngredientList = Cheeseburger;
		// ///////////////////////////////////////////////////////////////////////////////////////
		Message.DATA_MenuItem turkeysandwich = new Message.DATA_MenuItem(
				"TurkeySandwich", 11.00, 7, 0, 0);

		Message.DATA_Ingredient turkeysandwich1 = new Message.DATA_Ingredient(
				"CheeseSlice", 3);
		Message.DATA_Ingredient turkeysandwich2 = new Message.DATA_Ingredient(
				"Bun", 1);
		Message.DATA_Ingredient turkeysandwich3 = new Message.DATA_Ingredient(
				"Turkey", 1);
		Message.DATA_Ingredient turkeysandwich4 = new Message.DATA_Ingredient(
				"Lettuce", 1);

		ArrayList<Message.DATA_Ingredient> Turkey_Sandwich = new ArrayList<Message.DATA_Ingredient>();
		Turkey_Sandwich.add(turkeysandwich1);
		Turkey_Sandwich.add(turkeysandwich2);
		Turkey_Sandwich.add(turkeysandwich3);
		Turkey_Sandwich.add(turkeysandwich4);

		turkeysandwich.menuIngredientList = Turkey_Sandwich;

		// ///////////////////////////////////////////////////////////////////////////////////////
		Message.DATA_MenuItem baconburger = new Message.DATA_MenuItem(
				"BaconBurger", 12.00, 13, 0, 0);

		Message.DATA_Ingredient baconburger1 = new Message.DATA_Ingredient(
				"CheeseSlice", 3);
		Message.DATA_Ingredient baconburger2 = new Message.DATA_Ingredient(
				"Bun", 1);
		Message.DATA_Ingredient baconburger3 = new Message.DATA_Ingredient(
				"Beef", 1);
		Message.DATA_Ingredient baconburger4 = new Message.DATA_Ingredient(
				"Bacon", 1);

		ArrayList<Message.DATA_Ingredient> Bacon_Burger = new ArrayList<Message.DATA_Ingredient>();
		Bacon_Burger.add(baconburger1);
		Bacon_Burger.add(baconburger2);
		Bacon_Burger.add(baconburger3);
		Bacon_Burger.add(baconburger4);

		baconburger.menuIngredientList = Bacon_Burger;

		// ///////////////////////////////////////////////////////////////////////////////////////
		Message.DATA_MenuItem tunasandwich = new Message.DATA_MenuItem(
				"TunaSandwich", 10.00, 8, 0, 0);

		Message.DATA_Ingredient tunasandwich1 = new Message.DATA_Ingredient(
				"Lettuce", 3);
		Message.DATA_Ingredient tunasandwich2 = new Message.DATA_Ingredient(
				"Bun", 1);
		Message.DATA_Ingredient tunasandwich3 = new Message.DATA_Ingredient(
				"Tuna", 1);
		Message.DATA_Ingredient tunasandwich4 = new Message.DATA_Ingredient(
				"Tomatoes", 2);

		ArrayList<Message.DATA_Ingredient> Tuna_Sandwich = new ArrayList<Message.DATA_Ingredient>();
		Tuna_Sandwich.add(tunasandwich1);
		Tuna_Sandwich.add(tunasandwich2);
		Tuna_Sandwich.add(tunasandwich3);
		Tuna_Sandwich.add(tunasandwich4);

		tunasandwich.menuIngredientList = Tuna_Sandwich;

		// ///////////////////////////////////////////////////////////////////////////////////////
		Message.DATA_MenuItem bowtiepasta = new Message.DATA_MenuItem(
				"BowTiePasta", 10.00, 20, 0, 0);

		Message.DATA_Ingredient bowtiepasta1 = new Message.DATA_Ingredient(
				"WheatBowTiePasta", 3);
		Message.DATA_Ingredient bowtiepasta2 = new Message.DATA_Ingredient(
				"CheeseSlice", 1);
		Message.DATA_Ingredient bowtiepasta3 = new Message.DATA_Ingredient(
				"Spice", 1);
		// Message.DATA_Ingredient bowtiepasta4 = new
		// Message.DATA_Ingredient("Tomatoes", 2);

		ArrayList<Message.DATA_Ingredient> Bow_Tie_Pasta = new ArrayList<Message.DATA_Ingredient>();
		Bow_Tie_Pasta.add(bowtiepasta1);
		Bow_Tie_Pasta.add(bowtiepasta2);
		Bow_Tie_Pasta.add(bowtiepasta3);
		// Bow_Tie_Pasta.add(bowtiepasta4);

		bowtiepasta.menuIngredientList = Bow_Tie_Pasta;

		// ///////////////////////////////////////////////////////////////////////////////////////
		Message.DATA_MenuItem hersheycake = new Message.DATA_MenuItem(
				"HersheyCake", 4.00, 5, 0, 0);

		Message.DATA_Ingredient hersheycake1 = new Message.DATA_Ingredient(
				"ChocolateSprinkles", 3);
		Message.DATA_Ingredient hersheycake2 = new Message.DATA_Ingredient(
				"Oreos", 1);
		Message.DATA_Ingredient hersheycake3 = new Message.DATA_Ingredient(
				"WhipCream", 1);
		// Message.DATA_Ingredient hersheycake4 = new
		// Message.DATA_Ingredient("Tomatoes", 2);

		ArrayList<Message.DATA_Ingredient> Hershey_Cake = new ArrayList<Message.DATA_Ingredient>();
		Hershey_Cake.add(hersheycake1);
		Hershey_Cake.add(hersheycake2);
		Hershey_Cake.add(hersheycake3);
		// Hershey_Cake.add(hersheycake4);

		hersheycake.menuIngredientList = Hershey_Cake;

		// ///////////////////////////////////////////////////////////////////////////////////////
		Message.DATA_MenuItem roastbeef = new Message.DATA_MenuItem(
				"RoastBeef", 13.00, 15, 0, 0);

		Message.DATA_Ingredient roastbeef1 = new Message.DATA_Ingredient(
				"BeefStock", 3);
		Message.DATA_Ingredient roastbeef2 = new Message.DATA_Ingredient(
				"GarlicPieces", 1);
		Message.DATA_Ingredient roastbeef3 = new Message.DATA_Ingredient(
				"Salt/Pepper", 1);
		// Message.DATA_Ingredient roastbeef4 = new
		// Message.DATA_Ingredient("Tomatoes", 2);

		ArrayList<Message.DATA_Ingredient> Roast_Beef = new ArrayList<Message.DATA_Ingredient>();
		Roast_Beef.add(roastbeef1);
		Roast_Beef.add(roastbeef2);
		Roast_Beef.add(roastbeef3);
		// Roast_Beef.add(roastbeef4);

		roastbeef.menuIngredientList = Roast_Beef;

		ArrayList<Message.DATA_MenuItem> menuItemList = new ArrayList<Message.DATA_MenuItem>();
		menuItemList.add(burger);
		menuItemList.add(turkeysandwich);
		menuItemList.add(baconburger);
		menuItemList.add(tunasandwich);
		menuItemList.add(bowtiepasta);
		menuItemList.add(hersheycake);
		menuItemList.add(roastbeef);

		menu.menuItems = menuItemList;

		/*
		 * Network variables.
		 */
		Socket clientSocket = null;
		ServerSocket serverSocket;

		try {
			serverSocket = new ServerSocket(14000);

			while (true) {
				clientSocket = serverSocket.accept();
				System.out.println("Got a new message from "
						+ clientSocket.getRemoteSocketAddress());
				new Thread(new Controller(clientSocket, menu, inventory))
						.start();
			}

		} catch (IOException e) {

			e.printStackTrace();
		}

	}

}
